'''A module that writes something somewhere

Author(s):
    Carl Anderson (carl.anderson@weightwatchers.com)

'''
from primrose.base.node import AbstractNode

class AbstractWriter(AbstractNode):
    """write some data somewhere"""
    pass
